@@ -1,6 +1,9 @@
 import logging
 import os
+import queue
+import sys
 import time
+import traceback
 from multiprocessing import Process, Event
 from signal import SIGINT, SIGKILL
 from typing import Callable, Union, Type, Optional
@@ -27,6 +30,44 @@ class ExceptionProcess(Process):
             logging.exception(e)
 
 
+class MyMultiProcessingHandler(multiprocessing_logging.MultiProcessingHandler):
+    def _receive(self):
+        while True:
+            try:
+                if self._is_closed and self.queue.empty():
+                    break
+
+                record = self.queue.get(timeout=0.2)
+                self.sub_handler.emit(record)
+            except (KeyboardInterrupt, SystemExit):
+                raise
+            except (BrokenPipeError, EOFError):
+                break
+            except queue.Empty:
+                pass  # This periodically checks if the logger is closed.
+            except:
+                traceback.print_exc(file=sys.stderr)
+                break
+
+        self.queue.close()
+        self.queue.join_thread()
+
+
+def install_mp_handler(logger=None):
+    """Wraps the handlers in the given Logger with an MultiProcessingHandler.
+
+    :param logger: whose handlers to wrap. By default, the root logger.
+    """
+    if logger is None:
+        logger = logging.getLogger()
+
+    for i, orig_handler in enumerate(list(logger.handlers)):
+        handler = MyMultiProcessingHandler("mp-handler-{0}".format(i), sub_handler=orig_handler)
+
+        logger.removeHandler(orig_handler)
+        logger.addHandler(handler)
+
+
 class InterruptTestMixin(TestCase):
     """
     Helper class for testing interrupts and exceptions in code
@@ -70,7 +111,7 @@ class InterruptTestMixin(TestCase):
         """
         starts the process and waits until it is responsive by calling calls wait_for_setup()
         """
-        multiprocessing_logging.install_mp_handler()
+        install_mp_handler()
         self.process = ExceptionProcess(target=f)
         self.process.start()
         return self.process
