@@ -1,8 +1,10 @@
 import json
+from datetime import datetime
 from textwrap import dedent
 from unittest import TestCase
 
-from openmodule.models.backend import CountMessage, AccessRequest, Access, AccessCategory, check_recurrence
+from openmodule.models.backend import CountMessage, AccessRequest, Access, AccessCategory, check_recurrence, \
+    SessionStartMessage, SessionFinishMessage
 from openmodule.models.base import Gateway
 from openmodule.rpc.server import RPCServer
 from openmodule.utils.backend import Backend
@@ -274,6 +276,23 @@ class RpcTest(BackendTestMixin, RPCServerTestMixin, TestCase):
         transaction_id = self.create_count_message(direction=direction, gate=gate)
         self.zmq_client.send("count", **self.messages_by_transaction[transaction_id].dict())
 
+    def send_session_start_message(self, gate="gate1", medium_id="GARIVO1", medium_type="lpr", zone_id="zone_1",
+                                   user_id="user-1", timestamp=None, session_id="session-1"):
+        entry_data = dict(used_medium=dict(id=medium_id, type=medium_type), gate=gate)
+
+        message = SessionStartMessage(zone_id=zone_id, user_id=user_id, id=session_id, cost_table={}, serviceable=True,
+                                       entry_time=timestamp or datetime.utcnow(), entry_data=entry_data)
+        self.core.publish(message, b"session")
+
+    def send_session_finish_message(self, gate="gate2", medium_id="GARIVO1", medium_type="lpr", zone_id="zone_1",
+                                    user_id="user-1", timestamp=None, session_id="session-1"):
+        exit_data = dict(used_medium=dict(id=medium_id, type=medium_type), gate=gate)
+        message = SessionFinishMessage(zone_id=zone_id, user_id=user_id, id=session_id, cost_table={}, serviceable=True,
+                                       exit_time=timestamp or datetime.utcnow(), exit_data=exit_data, entry_data={},
+                                       entry_time=datetime.utcnow())
+
+        self.core.publish(message, b"session")
+
     def test_registration(self):
         # auto register on start
         message = self.zmq_client.wait_for_message_on_topic(topic=b"backend", timeout=5)
@@ -315,6 +334,30 @@ class RpcTest(BackendTestMixin, RPCServerTestMixin, TestCase):
         self.backend.wait_for_message_process()
         self.assertEqual(0, self.backend.checked_in)
 
+    def test_session_check_in(self):
+        self.send_session_start_message()
+        self.backend.wait_for_message_process()
+        self.assertEqual(1, self.backend.checked_in)
+
+        MockEvent.reset_all_mocks()
+        with self.assertLogs() as cm:
+            self.send_session_start_message(gate="error")
+            self.backend.wait_for_message_process()
+        self.assertEqual(1, self.backend.checked_in)
+        self.assertIn("Error in session check in", cm.output[0])
+
+    def test_session_check_out(self):
+        self.send_session_finish_message()
+        self.backend.wait_for_message_process()
+        self.assertEqual(1, self.backend.checked_in)
+
+        MockEvent.reset_all_mocks()
+        with self.assertLogs() as cm:
+            self.send_session_finish_message(gate="error")
+            self.backend.wait_for_message_process()
+        self.assertEqual(1, self.backend.checked_in)
+        self.assertIn("Error in session check out", cm.output[0])
+
     def test_datetime_is_converted_to_unix_timestamp(self):
         gateway = Gateway(gate="entry", direction="in")
         request = AccessRequest(gateway=gateway, name=self.core.config.NAME, medium_type="lpr", id="GTEST1")
